/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 * $Id$
 *
 * Copyright (C) 2002-01-15 Psyco Dedman
 *
 * Modified to integrate into ROCKbox! OS by Nicolas Sauzede.
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/

#include "button.h"
#include "lcd.h"
#include "led.h"
#include "kernel.h"

#include "gamebox.h"

#ifdef HAVE_NEW_CHARCELL_LCD

#define B0 0xf0
#define BR 0xfd
#define BT 0xf7
#define FE 0xA0
#define AU 0x18
#define AD 0x19
#define AL 0x7F
#define AR 0x7E
#define EX 0x90

static const unsigned char player[] = { 0x11, 0x1F, 0x10, 0x1E };
#define PLAY 0x10

#else

#define B0 0x8B
#define BR 0x8B
#define BT 0xDB
#define FE 0x24
#define AU 0x86
#define AD 0x87
#define AL 0x40
#define AR 0x42
#define EX 0x2E

static const unsigned char player[] = { 0x88, 0x86, 0x89, 0x87 };
#define PLAY 0x89

#endif

// This is the map
// anything over B0 is considered solid for collision detection
// anything under can be walked through
// EX marks map end...
// this will all be changed once i figure out custom icons...
// FIXME: make maps dynamically generated, or at least work out a level system
static unsigned char mapa[][25] = {
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR },
	{ BR,FE,BR,BR,BR,BR,FE,FE,FE,FE,FE,FE,FE,FE,BR,FE,FE,FE,FE,FE,FE,BR,EX,BR },
	{ BR,FE,FE,FE,FE,FE,FE,BR,FE,FE,BT,BR,BT,FE,FE,FE,BT,BR,BR,BT,FE,BR,FE,BR },
	{ BR,BR,BR,BR,FE,FE,FE,FE,BR,FE,FE,FE,FE,FE,BR,FE,FE,FE,FE,FE,FE,BR,FE,BR },
	{ BR,FE,FE,FE,FE,BR,BR,FE,FE,FE,BT,BT,BR,FE,FE,FE,BR,BR,BR,BR,FE,BR,FE,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,BR,BR,BR,FE,BR,BR,FE,FE,FE,FE,BR,FE,FE,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,BR,FE,FE,FE,BR,BR,FE,BR,BR,FE,BR,FE,BR,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,BR,FE,BR,BR,BR,BR,FE,BR,FE,FE,BR,FE,BR,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,BR,FE,FE,FE,BR,FE,FE,BR,FE,BR,BR,FE,FE,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,FE,BR,BR,FE,FE,FE,FE,BR,FE,BR,BR,BR,FE,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,FE,FE,BR,BR,BR,BR,BR,FE,FE,FE,FE,AU,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,FE,FE,FE,FE,FE,BR,BR,BR,BR,BR,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,FE,FE,FE,FE,AL,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR },
};

int mapasize = sizeof( mapa[0]);

// prints string directly to LCD without ascii charset conversion
// y is the line to print on, xoff is how much to offset on the x axis
// 	(for horizontal scrolling)
// string is the string to be printed, len is it's length
void lcd_printmap(char y, char xoff, unsigned char* string, int len)
{
	int i;
	lcd_gotoxy(0,y);
	for (i=0; string[i] && i<len; i++)
		lcd_putbyte(string[i+xoff]);
}

void gamebox_loop(void)
{
	bool the_led = false;
	int x=1,y=1,j,xo=1,yo=1;
	int mapx=0,mapy=0,dir=0,pdir=2;

    while(1)
    {
		lcd_printmap(0,mapx,mapa[mapy+dir],11);
		lcd_printmap(1,mapx,mapa[mapy+1+dir],11);
		lcd_gotoxy( xo,yo);
		lcd_putbyte( player[pdir]);
		
        switch (button_get(true))
        {
        	case BUTTON_ON:
        		return;
        	case BUTTON_UP:
				if (mapa[y-1][x]<=B0)
					y--;
				dir=1;
				pdir=1;
        		break;
#ifdef HAVE_PLAYER_KEYPAD
        	case BUTTON_MENU:
#endif
        	case BUTTON_DOWN:
				if (mapa[y+1][x]<=B0)
					y++;
				dir=0;
				pdir=3;
        		break;
        	case BUTTON_LEFT:
				if (mapa[y][x-1]<=B0)
					x--;
				pdir=0;
        		break;
        	case BUTTON_RIGHT:
				if (mapa[y][x+1]<=B0)
					x++;
				pdir=2;
        		break;
        }

		if ((x>5) && (x<(mapasize-6))) {
			xo=5;
			mapx=x-5;
		} else { xo=x; }
		if (x<=5) {
			mapx=0;
		}
		if (x>=(mapasize-6)) {
			xo=x-(mapasize-12);
			mapx=mapasize-12;
		}
		if (dir>=1) {
			yo=1;
			//mapy=y-1;
			mapy=y-2;
		}
		if (dir<1) {
			yo=0;
			mapy=y;
		}

		if (mapa[y][x] == EX)
		{
			int duty = 4;
			
			the_led = true;
			lcd_puts(0,0,"You  Win!!!");
			lcd_puts(0,1,"   YAY!!   ");
			for (j = 0; j < duty; j++)
			{
				led( the_led = !the_led);
				sleep( HZ / duty);
			}
			break;
		}
    }
}

Menu gamebox( void)
{
	lcd_clear_display();
    lcd_puts( 0, 0, "* G4m3B0x *");
#ifdef HAVE_NEW_CHARCELL_LCD
    lcd_double_height( true);
#endif
	sleep( HZ);
	lcd_clear_display();
#ifdef HAVE_NEW_CHARCELL_LCD
    lcd_double_height( false);
#endif

    gamebox_loop();

    return MENU_OK;
}
